/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <gtk/gtk.h>

#include "psensor.h"


int compute_y(float temp, int mint, int maxt, int width, int height) 
{
	double t = temp - mint;
	return height - ((double)height * ( t / (maxt-mint)));
}

void update_graph(struct psensor** sensors, GtkWidget* w_graph) 
{
	int mint = get_min_temp(sensors);
	int maxt = get_max_temp(sensors);


	int width = w_graph->allocation.width;
	int height = w_graph->allocation.height;

	// start offset of the graph
	// 0..g_off used to print max/min T
	int g_xoff = 50;
	// the width of the graph
	int g_width = width - g_xoff;

	// vertical padding
	int g_ypad = 4; 
	int g_yoff = g_ypad;
	int g_height = height - 2*g_ypad;

	cairo_t *cr = gdk_cairo_create(w_graph->window);

	if (!cr) return ;


	// graph background
	cairo_set_source_rgb(cr, 0.91, 0.91, 0.66); // sand yellow
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_fill (cr);


	// setup dash style
	double dashes[] = {1.0,  /* ink */
                   2.0,  /* skip */
                   1.0,  /* ink */
                   2.0   /* skip*/
                  };
	int    ndash  = sizeof (dashes)/sizeof(dashes[0]);
	cairo_set_dash (cr, dashes, ndash, 0);
	cairo_set_line_width (cr, 1);
	cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);


	// vertical lines representing time steps
	int i=0;	
	for(i = 0; i <= 5; i++) {

		int x =  i * (g_width / 5) + g_xoff;
		cairo_move_to(cr,x,g_yoff);
		cairo_line_to(cr,x,g_height+g_yoff);
		cairo_stroke(cr);
	}

	// horizontal lines
	// draws a line for each 10C step
	for(i = mint; i < maxt; i++) {
		if (i % 10 == 0) {
			int y = compute_y(i, mint, maxt, width, g_height);
			cairo_move_to(cr,g_xoff,y+g_yoff);
			cairo_line_to(cr,g_xoff+g_width,y+g_yoff);
			cairo_stroke(cr);
		}
	}

	// back to normal line style
	cairo_set_dash (cr, 0, 0, 0);

	// print min and max temp in the canvas
	char strmin[200];
	char strmax[200];
	sprintf(strmin,"+%dC",mint);
	sprintf(strmax,"+%dC",maxt);
	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 1.0);
	cairo_text_extents_t te;
	cairo_select_font_face (cr, "sans-serif",
				CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
	cairo_set_font_size (cr, 10.0);
	cairo_text_extents (cr, strmax, &te);
	cairo_move_to(cr,g_xoff - te.width - 5,te.height+g_yoff);
	cairo_show_text(cr, strmax);

	cairo_text_extents (cr, strmin, &te);
	cairo_move_to(cr,g_xoff - te.width - 5,height-te.height/2-g_ypad);
	cairo_show_text(cr, strmin);

	// .. and finaly draws the temperature graphs
	struct psensor** sensor_cur = sensors;
	while(*sensor_cur) {
		if ((*sensor_cur)->enabled) {
			cairo_set_source_rgba(cr, 
					      (*sensor_cur)->color[0], 
					      (*sensor_cur)->color[1], 
					      (*sensor_cur)->color[2], 
					      0.9); // alpha
			cairo_set_line_width (cr, 1);
			
			int first = 1;
			int i;
			
			for(i = 0 ; i < MAX_TEMPS; i++) {
				if (!((*sensor_cur)->temps[i])) 
					continue;				
				
				int x = i * g_width / MAX_TEMPS + g_xoff;
				int y = compute_y((*sensor_cur)->temps[i],
						  mint, maxt,
						  g_width, g_height)+g_yoff;
				
				if (first) {
					cairo_move_to(cr, x, y);
					first = 0;
				} else {
					cairo_line_to(cr, x, y);
				}
				
			}
			cairo_stroke(cr);
		}

		sensor_cur++;
	}

	cairo_destroy(cr);
}
